-- wTaskNoteFrame.lua: Sep 2010
-- Control and design of the list of notes frame
--[[

Copyright (C) 2010, WuphonsReach

License:

	This file is part of wTaskNote.
	
	wTaskNote is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	wTaskNote is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with wTaskNote.  If not, see <http://www.gnu.org/licenses/>.

Notes:

	All World or Warcraft game related content and images are the property
	of Blizzard Entertainment, Inc. and protected by U.S. and international
	copyright laws. The Addon (code and supporting files) is property of
	the author and protected by U.K. and international copyright laws.

]]

-- Localization, works in conjunction with the wowace.com website
-- Gets us a reference to the current locale table created by AceLocale
local L = LibStub("AceLocale-3.0"):GetLocale("wTaskNote")

-- stub in AceGUI so we can make use of some features
local AceGUI = LibStub("AceGUI-3.0")

-- General design is going to be very similar to that of the current Blizzard quest frame
-- A tree in the left-hand pane, with the content of the note in the right-hand pane

-- Left-hand tree headers based on AceDB3 subtables: 
--   "global", "realm", "class", "race", "faction", "factionrealm", etc.
-- Global notes come first in the list, followed by current realm, current faction, current char
-- Probably won't use all of the available subtables, being able to share notes based on:
--   "global", "current realm", "current faction", "current char", "current profile" will be enough.
-- Design question is whether we want to allow one character to access another character's notes (probably yes)
-- If we go with global accessibility, then we only use the "global" level of AceDB, and we end up
-- with a larger amount of data loaded into memory.
--
-- Possible tree design
-- [+]Global
--   [+]Category 1
--     note1
--     note3
--   [+]Category 2
--     note2
--     note4
-- [+]Current Profile
--   [+]Category 1
--     note1
--     note3
--   [+]Category 2
--     note2
--     note4
-- [+]Current Server
--   [+]Category 1
--     note1
--     note3
--   [+]Category 2
--     note2
--     note4
-- [+]Current Character
--   [+]Category 1
--     note1
--     note3
--   [+]Category 2
--     note2
--     note4
-- 
-- As we click on stuff in the left pane, we need to update the right pane to display the contents
-- Buttons: 
--   "New" (new note) 
--   "Delete" (delete selected note) 
--   "Move" (move a note to a different category, functionality might go on the edit frame instead)
--   "Copy" (make a copy of the currently selected note)
-- Note Attributes:
--   "id" (random identifier string/number)
--   "scope"  (global, server, character)
--   "category" (determines where in the tree it appears)
--   "subject" (short subject line)
--   "body" (body, not sure on length limit, probably same as a mail body)
--
-- It will probably make more sense that notes are always in "edit" mode.  In which case
-- we'll mostly just need a "Save" / "Undo" pair of buttons on the right side panel at the bottom.
--
-- Category is going to need a text entry pop-up box if the user clicks "New" next to the category drop-down
--
-- Right-hand pane UI elements:
--
-- Scope: [drop-down w/ global, server, char, default of char]
--   if scope changes, then category also changes to the first category under that scope
-- Category: [drop-down, default of "General"] w/ [New] button
--   if category changes, then we need to move the message in the left-hand list
-- Subject: [text box, same limit as mail message]
--   current limit: 64 characters in mail message subject
-- Body: (unlabled control)
--   current limit: 500 characters in mail body
-- Buttons: [DELETE] [SAVE] [CANCEL]
-- 
-- Left-hand pane UI elements:
-- 
-- Tree view of scope / category / notes
--   sort by current subject line
-- Buttons: [DELETE] [NEW] [COPY]
-- 

-----------------------------------------------------------------------------
-- Create the main frame of the interface (for notes)
-- Look at Auctioneer's SearchMain.lua file: lib.CreateAuctionFrames()
-- We'll probably base our artwork off of Blizzard artwork
function wTaskNote:CreateNoteFrame()
	if wTaskNoteFrame then return end
	
	-- Create the master UI frame for note editing/display
	-- Note: also creates a global variable named wTaskNoteFrame as a side-effect
	local f = CreateFrame("Frame", "wTaskNoteFrame", UIParent)
	f:Hide() -- always hide the frame at the start after creation
	--f:SetFrameStrata("DIALOG")
	f:SetWidth(682)
	f:SetHeight(447)
	f:SetPoint("TOPLEFT", UIParent, 150, -100) -- controls starting position of the frame
	f:SetMovable(true) -- frame is movable
	f:SetResizable(false) -- frame is not resizable (for now)
	f:SetClampedToScreen(true) -- keep our window constrained to the screen
	
	-- create an invisible mover frame
	--local header = f:CreateTexture(nil, "OVERLAY")
    --header:SetWidth(682)
    --header:SetHeight(25)
    --header:SetPoint("TOP", f, 'TOP', 0, -15)
    	
	--local mover = CreateFrame("Frame", nil, f)
	--mover:SetAllPoints(header)
	--f:EnableMouse(true) -- get mouse input (makes the frame clickable, otherwise it's transparent to mouse input)
    --mover:RegisterForDrag("LeftButton")
    --mover:SetScript("OnDragStart", function()		
    --	if(arg1 == "LeftButton") then			
    --  		t:StartMoving();
    --   	end
    --end)
    --mover:SetScript("OnDragStop", function()
    --	t:StopMovingOrSizing()
    --end) 
	--f:SetTopLevel(true) -- set ourselves to top level when clicked
	
	-- Layout the left/right background textures
	-- We don't bother giving CreateTexture() a name since we don't plan on referring
	-- back to this texture after this and we'll do all our work with a local reference
	local t_bgleft = f:CreateTexture(nil, "BORDER")
	t_bgleft:SetTexture("Interface\\Addons\\wTaskNote\\Textures\\wTaskNoteFrame-Left")
    t_bgleft:SetWidth(512)
    t_bgleft:SetHeight(445)
    t_bgleft:SetPoint("TOPLEFT", f)
    t_bgleft:SetTexCoord(0.0, 1.0, 0.0, 0.86914) -- magic numbers lifted from Blizzard's QuestLogFrame.xml file
	
	-- Layout the left/right background textures
	local t_bgright = f:CreateTexture(nil, "BORDER")
	t_bgright:SetTexture("Interface\\Addons\\wTaskNote\\Textures\\wTaskNoteFrame-Right")
    t_bgright:SetWidth(170)
    t_bgright:SetHeight(445)
    t_bgright:SetPoint("TOPRIGHT", f)
    t_bgright:SetTexCoord(0.0, 0.6640625, 0.0, 0.86914)

	-- Icon in upper left, should be the same as the minimap icon
	-- See: http://wowprogramming.com/utils/artbrowser/QuestFrame/UI-QuestLog-BookIcon.png
	-- TODO: Possibly provide a few different icons for users to pick from
	local t_icon = f:CreateTexture(nil, "BACKGROUND")
	t_icon:SetTexture("Interface\\Addons\\wTaskNote\\Textures\\wTaskNoteFrame-INVLetter06")
	t_icon:SetWidth(64)
	t_icon:SetHeight(64)
	t_icon:SetPoint("TOPLEFT", f, 3, -4)
	
	-- See: http://www.wuphonsreach.org/Games/WoW/wTaskNote/wTaskNote-rev-49.jpg
	-- Which contains a Screenshot of what revision #49 looks like, it includes all of the elements up until this point.
	
	-- Upper right frame "X" button, since we inherit from UIPanelCloseButton there's very little to do
	local b_frclose = CreateFrame("Button", nil, f, "UIPanelCloseButton")
	b_frclose:SetPoint("TOPRIGHT", f, 2, -8)
	
	-- Lower right "close" button
	local b_close = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
	b_close:SetWidth(80)
	b_close:SetHeight(22)
	b_close:SetPoint("BOTTOMRIGHT", f, -7, 14)
	b_close:SetScript("OnClick", function() wTaskNoteFrame:Hide() end)
	b_close:SetText(L["BtnClose"])
	
	-- Title FontString (title across the top of the window), L["UINoteTitle"]
    local txt_title = f:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    txt_title:SetPoint("TOP", f, 0, -17)
    txt_title:SetWidth(300)
    txt_title:SetHeight(14)
    txt_title:SetText(L["UINoteTitle"])
    
	-- Lower right-side buttons (delete )
	local b_delete = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
	b_delete:SetWidth(80)
	b_delete:SetHeight(22)
	b_delete:SetPoint("BOTTOMRIGHT", f, -250, 14)
	--b_delete:SetScript("OnClick", function() wTaskNoteFrame:Hide() end)
	b_delete:SetText(L["BtnDelete"])



    
	-- Hook into the OnShow/OnHide handler for our widget and add some custom
	-- code to the mix (such as playing a sound effect like the quest log does)
	f:SetScript("OnShow", wTaskNoteFrame_OnShow)
	f:SetScript("OnHide", wTaskNoteFrame_OnHide)
	--f:SetScript("OnEvent",wTaskNoteFrame_OnEvent)
	--f:SetScript("OnUpdate",wTaskNoteFrame_OnUpdate)
	
	-- Note: our frame should initially be hidden and Show() gets called in ToggleNoteFrame so don't call f:Show()
	--f:Show() -- do not enable this line
	
	
	--wTaskNoteFrame = AceGUI:Create("Frame")
	--wTaskNoteFrame:SetTitle("Example Frame")
	--wTaskNoteFrame:SetStatusText("AceGUI-3.0 Example Container Frame")
	--wTaskNoteFrame:SetWidth(800)
	--wTaskNoteFrame:SetHeight(400)
	--wTaskNoteFrame:SetCallback("OnClose", function(widget) widget:Hide() end)

	-- Setup the left frame and right frame
	--local leftframe = AceGUI:Create("ScrollFrame")
	--wTaskNoteFrame:AddChild(leftframe)
	--local rightframe = AceGUI:Create("ScrollFrame")
	--wTaskNoteFrame:AddChild(rightframe)
	
	-- Applying this layout will cause the first two children to take up half the width and full height
	--wTaskNoteFrame:SetLayout("wTaskNote_Split")

	--local editline = AceGUI:Create("EditBox")
	--editbox:SetLabel("Insert text:")
	--editbox:SetWidth(200)
	--rightframe:AddChild(editbox)
	
	--local editline = AceGUI:Create("EditBox")
	--editbox:SetLabel("Insert text:")
	--editbox:SetWidth(200)
	--rightframe:AddChild(editbox)

end

-----------------------------------------------------------------------------
-- Toggle visibility of the note frame
function wTaskNote:ToggleNoteFrame()
	if not wTaskNoteFrame then wTaskNote:CreateNoteFrame() end
	if wTaskNoteFrame:IsShown() then
		wTaskNoteFrame:Hide()
	else
		wTaskNoteFrame:Show()
	end
end

-----------------------------------------------------------------------------
-- Event that fires when the wTaskNoteFrame is displayed
-- The actual function name doesn't really matter, as long as it matches up
-- with the SetScript() call back where we created the frame.
function wTaskNoteFrame_OnShow(self)
	-- Play the "open quest log" sound
	PlaySound("igQuestLogOpen");
	
	-- DEBUG: walk our database tables (and sub-tables)
	wTaskNote:DebugWalkTable(wTaskNote.db.global.notes, "wTaskNote.db.global.notes")
	wTaskNote:DebugWalkTable(wTaskNote.db.realm.notes, "wTaskNote.db.realm.notes")
	wTaskNote:DebugWalkTable(wTaskNote.db.char.notes, "wTaskNote.db.char.notes")
	wTaskNote:DebugDisplayNoteCategories(wTaskNote.db.global.notes, "wTaskNote.db.global.notes")
end

-----------------------------------------------------------------------------
-- Event that fires when the wTaskNoteFrame is hidden
function wTaskNoteFrame_OnHide(self)
	-- Play the "close quest log" sound
	PlaySound("igQuestLogClose");
end

-----------------------------------------------------------------------------
-- Event that fires when the wTaskNoteFrame receives an event
-- See: http://www.wowwiki.com/Events_%28API%29
-- See: http://www.wowwiki.com/Handling_events
--function wTaskNoteFrame_OnEvent(self, event, ...)
--end

-----------------------------------------------------------------------------
-- Event that fires when the wTaskNoteFrame is updated
-- game engine will call your OnUpdate function once each frame
-- See: http://www.wowwiki.com/HOWTO:_Use_OnUpdate_Correctly
-- See: http://www.wowwiki.com/UIHANDLER_OnUpdate
--function wTaskNoteFrame_OnUpdate(self, elapsed)
--end